@@ -25,6 +25,8 @@ pub struct Network {
 
     // Network bus
     request_bus: Sender<Option<RpcOnWire>>,
+    // Server thread pool,
+    server_pool: tokio::runtime::Runtime,
 
     // Closing signal.
     keep_running: bool,
@@ -82,8 +84,9 @@ impl Network {
     pub fn add_server<S: Into<ServerIdentifier>>(
         &mut self,
         server_name: S,
-        server: Server,
+        mut server: Server,
     ) {
+        server.use_pool(self.server_pool.handle().clone());
         self.servers.insert(server_name.into(), Arc::new(server));
     }
 
@@ -264,6 +267,8 @@ impl Network {
         mark_trace!(rpc.trace, served);
     }
 
+    const WORKER_THREADS: usize = 16;
+
     pub fn run_daemon() -> Arc<Mutex<Network>> {
         let (network, rx) = Network::new();
 
@@ -272,6 +277,7 @@ impl Network {
 
         // Using tokio instead of futures-rs, because we need timer futures.
         let thread_pool = tokio::runtime::Builder::new_multi_thread()
+            .worker_threads(Self::WORKER_THREADS)
             .thread_name("network")
             .enable_time()
             .build()
@@ -325,6 +331,11 @@ impl Network {
     }
 
     fn new() -> (Self, Receiver<Option<RpcOnWire>>) {
+        // Server thread pool
+        let server_pool = tokio::runtime::Builder::new_multi_thread()
+            .thread_name("server-pool")
+            .build()
+            .expect("Creating server thread pool should not fail");
         // The channel has infinite buffer, could OOM the server if there are
         // too many pending RPCs to be served.
         let (tx, rx) = crossbeam_channel::unbounded();
@@ -335,6 +346,7 @@ impl Network {
             clients: Default::default(),
             servers: Default::default(),
             request_bus: tx,
+            server_pool,
             keep_running: true,
             stopped: Default::default(),
             rpc_count: std::cell::Cell::new(0),
@@ -635,7 +647,7 @@ mod tests {
         let (network, _) = make_network_and_client();
         let barrier = Arc::new(Barrier::new(THREAD_COUNT + 1));
         const THREAD_COUNT: usize = 200;
-        const RPC_COUNT: usize = 100;
+        const RPC_COUNT: usize = 10000;
 
         let mut handles = vec![];
         for i in 0..THREAD_COUNT {
@@ -656,6 +668,9 @@ mod tests {
                     results.push(reply);
                 }
                 for result in results {
+                    // Futures executor is used instead of tokio executor,
+                    // because it uses std::thread::park(), which is more
+                    // straightforward and faster than mutex + condvar.
                     futures::executor::block_on(result)
                         .expect("All futures should succeed");
                 }
@@ -668,6 +683,10 @@ mod tests {
             handle.join().expect("All threads should succeed");
         }
         eprintln!("Many requests test took {:?}", now.elapsed());
+        eprintln!(
+            "The average RPC latency is {:?}",
+            now.elapsed() / (RPC_COUNT as u32)
+        );
     }
 
     // A typical request takes about 80ms.
